# -*- coding: utf-8 -*-

from Queue import Queue
from netkit.contrib.tcp_client import TcpClient
import time
from thread import start_new_thread
from ..gw_box import GWBox
from ..gw_pb2 import RspToUsers, CloseUsers
from ..log import logger
from .. import constants


class Trigger(object):

    box_class = None
    msg_queue = None

    client = None

    def __init__(self, box_class, host, port, maxsize=-1):
        self.box_class = box_class
        self.msg_queue = Queue(maxsize)

        self.client = TcpClient(GWBox, host, port)

    def start(self):
        start_new_thread(self._send_worker, ())
        start_new_thread(self._recv_worker, ())

    def write_to_worker(self, data):
        """
        透传到worker进行处理
        """

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_TRIGGER_REQ

        if isinstance(data, self.box_class):
            # 打包
            data = data.pack()
        elif isinstance(data, dict):
            data = self.box_class(data).pack()
            
        gw_box.body = data

        msg = gw_box.pack()

        try:
            self.msg_queue.put_nowait(msg)
            return True
        except Exception, e:
            logger.error('msg_queue.put_nowait fail. msg: %s', msg, exc_info=True)
            return False

    def write_to_users(self, data_list):
        """
        格式为
        [(uids, box), ...]
        :param data_list:
        :return:
        """

        msg = RspToUsers()

        for uids, data in data_list:
            if isinstance(data, self.box_class):
                data = data.pack()
            elif isinstance(data, dict):
                data = self.box_class(data).pack()

            row = msg.rows.add()
            row.buf = data
            row.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_WRITE_TO_USERS
        gw_box.body = msg.SerializeToString()

        msg = gw_box.pack()

        try:
            self.msg_queue.put_nowait(msg)
            return True
        except Exception, e:
            logger.error('msg_queue.put_nowait fail. msg: %s', msg, exc_info=True)
            return False

    def close_users(self, uids):
        msg = CloseUsers()
        msg.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_CLOSE_USERS
        gw_box.body = msg.SerializeToString()

        msg = gw_box.pack()

        try:
            self.msg_queue.put_nowait(msg)
            return True
        except Exception, e:
            logger.error('msg_queue.put_nowait fail. msg: %s', msg, exc_info=True)
            return False


    def _send_worker(self):
        """
        处理发送消息
        :return:
        """
        def inner_func():
            # 永久等在这里就可以
            msg = self.msg_queue.get()

            while self.client.closed():
                # 要一直等到可以发送为止，否则消息会被丢失
                logger.error('connection closed, sleep %ss', constants.TRY_CONNECT_INTERVAL)
                time.sleep(constants.TRY_CONNECT_INTERVAL)

            ret = self.client.write(msg)
            if not ret:
                logger.error('client.write fail. ret: %s, msg: %s', ret, msg)

        while 1:
            try:
                inner_func()
            except KeyboardInterrupt:
                break
            except:
                # 其他情况都应该继续
                continue

    def _recv_worker(self):
        """
        recv负责判断是否closed，以及重连
        :return:
        """
        def inner_func():
            while self.client.closed():
                try:
                    self.client.connect()
                except KeyboardInterrupt, e:
                    # 中断
                    raise e
                except:
                    # 说明链接失败了
                    logger.error('connect fail, sleep %ss', constants.TRY_CONNECT_INTERVAL)
                    time.sleep(constants.TRY_CONNECT_INTERVAL)

            # 说明这里是对的
            ret = self.client.read()
            if not ret:
                # 链接断开
                self.client.close()

        while 1:
            try:
                inner_func()
            except KeyboardInterrupt:
                break
            except:
                # 其他情况都应该继续
                continue
