# -*- coding: utf-8 -*-

import numbers
from ..gw_box import GWBox
from ..log import logger
from .. import constants
from ..gw_pb2 import RspToUsers, CloseUsers


class Request(object):
    """
    请求
    """

    conn = None
    gw_box = None
    box = None
    is_valid = False
    blueprint = None
    blueprint_name = None
    blueprint_cmd = None

    def __init__(self, conn, gw_box):
        self.conn = conn
        self.gw_box = gw_box
        # 赋值
        self.is_valid = self._parse_raw_data()

    @property
    def app(self):
        return self.conn.app

    def _parse_raw_data(self):
        if not self.gw_box.body:
            return True

        try:
            self.box = self.app.box_class()
        except Exception, e:
            logger.error('parse raw_data fail. e: %s, request: %s', e, self)
            return False

        if self.box.unpack(self.gw_box.body) > 0:
            self._parse_blueprint_info()
            return True
        else:
            logger.error('unpack fail. request: %s', self)
            return False

    def _parse_blueprint_info(self):
        if self.cmd is None:
            return

        cmd_parts = str(self.cmd).split('.')
        self.blueprint_name, self.blueprint_cmd = cmd_parts if len(cmd_parts) == 2 else (None, self.cmd)

        for bp in self.app.blueprints:
            if self.blueprint_name == bp.name or isinstance(self.blueprint_cmd, numbers.Number):
                # blueprint name 匹配; 或者cmd是数字类型，即与blueprint name无关
                if bp.get_route_view_func(self.blueprint_cmd):
                    self.blueprint = bp
                    break

    @property
    def cmd(self):
        try:
            return self.box.cmd
        except:
            return None

    def write_to_client(self, data):
        """
        写回
        :param data: 可以是dict也可以是box
        :return:
        """
        if isinstance(data, self.app.box_class):
            data = data.pack()
        elif isinstance(data, dict):
            data = self.box.map(data).pack()

        gw_box = self.gw_box.map(dict(
            cmd=constants.CMD_WRITE_TO_CLIENT,
            body=data,
        ))

        return self.conn.write(gw_box.pack())

    def write_to_users(self, data_list):
        """
        格式为
        [(uids, box), ...]
        :param data_list:
        :return:
        """

        msg = RspToUsers()

        for uids, data in data_list:
            if isinstance(data, self.app.box_class):
                data = data.pack()
            elif isinstance(data, dict):
                data = self.box_class(data).pack()

            row = msg.rows.add()
            row.buf = data
            row.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_WRITE_TO_USERS
        gw_box.body = msg.SerializeToString()

        self.conn.write(gw_box.pack())

    def close_client(self):
        gw_box = self.gw_box.map(dict(
            cmd=constants.CMD_CLOSE_CLIENT,
        ))

        return self.conn.write(gw_box.pack())

    def close_users(self, uids):
        msg = CloseUsers()
        msg.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_CLOSE_USERS
        gw_box.body = msg.SerializeToString()

        self.conn.write(gw_box.pack())

    def login_client(self, uid):
        gw_box = self.gw_box.map(dict(
            cmd=constants.CMD_LOGIN_CLIENT,
            uid=uid,
        ))

        return self.conn.write(gw_box.pack())

    def logout_client(self):
        gw_box = self.gw_box.map(dict(
            cmd=constants.CMD_LOGOUT_CLIENT,
        ))

        return self.conn.write(gw_box.pack())

    def __repr__(self):
        return 'cmd: %r, gw_box: %r' % (self.cmd, self.gw_box)
